@@ -14,19 +14,24 @@
  *   (at your option) any later version.                                   *
  *                                                                         *
  ***************************************************************************/
-// $Id: ClanBomber.cpp,v 1.49 2000/03/28 16:27:38 andi Exp $
+// $Id: ClanBomber.cpp,v 1.12 2004/02/16 18:32:27 der_fisch Exp $
 
 #include <ClanLib/Core/System/system.h>
+#include <ClanLib/Display/setupdisplay.h>
+#include <ClanLib/Sound/setupsound.h>
 #include <ClanLib/Core/System/setupcore.h>
 #include <ClanLib/Core/System/error.h>
-#include <ClanLib/Core/Sound/soundbuffer.h>
-#include <ClanLib/Core/Input/input.h>
-#include <ClanLib/Core/Input/inputbuffer.h>
-#include <ClanLib/Core/Input/keyboard.h>
-#include <ClanLib/Core/Display/display.h>
-#include <ClanLib/Core/Display/surface.h>
+#include <ClanLib/Core/Resources/datafile_compiler.h>
+#include <ClanLib/Sound/soundbuffer.h>
+#include <ClanLib/Display/Input/input.h>
+#include <ClanLib/Display/Input/inputbuffer.h>
+#include <ClanLib/Display/Input/keyboard.h>
+#include <ClanLib/Display/Input/key.h>
+#include <ClanLib/Display/Display/display.h>
+#include <ClanLib/Display/Display/surface.h>
 #include <ClanLib/Core/IOData/inputsource.h>
-#include <ClanLib/Core/Font/font.h>
+#include <ClanLib/Display/Font/font.h>
+#include <ClanLib/display.h>
 
 #include "ClanBomber.h"
 
@@ -63,42 +68,67 @@ CL_String ClanBomberApplication::local_map_path;
 
 char *ClanBomberApplication::get_title()
 {
-	return "ClanBomber 1.01a";
+	return "ClanBomber 1.05";
 }
 
-void ClanBomberApplication::init_modules()
+void ClanBomberApplication::quit_app(int value)
 {
-	CL_SetupCore::init();
-}
-
-void ClanBomberApplication::deinit_modules()
-{
-	CL_SetupCore::deinit();
+	deinit_modules();
+	exit(value);
 }
 
 int ClanBomberApplication::main(int argc, char** argv)
 {
+
 	try
 	{
-		CL_SetupCore::init_display();
-		CL_SetupCore::init_sound();
-     	
-     	srand( (long)time(NULL) );
+		init_modules();
 
-     	CL_Display::set_videomode(800, 600, 16);
+		bool fullscreen = false;
+#ifdef WIN32
+		fullscreen = true;
+#endif
 
-     	CL_Display::clear_display();
-     	CL_Display::flip_display();
-     	CL_Display::clear_display();
-     	CL_Display::flip_display();
+		if (argc > 1) {
+			if (strcmp (argv[1], "-datafile") == 0) {
+				CL_DatafileCompiler::write("clanbomber.scr", "clanbomber.dat");
+				quit_app(0);
+			}
+			else if (strcmp (argv[1], "-fullscreen") == 0) {
+				fullscreen = true;
+			}
+			else if (strcmp (argv[1], "-window") == 0) {
+				fullscreen = false;
+			}
+			else if (strcmp (argv[1], "-h") == 0 ||
+					strcmp (argv[1], "-help") == 0 ||
+					strcmp (argv[1], "--help") == 0 ||
+					strcmp (argv[1], "-?") == 0 ||
+					strcmp (argv[1], "/?") == 0 ||
+					strcmp (argv[1], "/h") == 0) {
+				cout << "usage: clanbomber [-fullscreen | -window]" << endl;
+				quit_app(0);
+			}
+		}
+
+		srand( (long)time(NULL) );
+
+		CL_Display::set_videomode(800, 600, 16, fullscreen);
+
+		CL_Display::clear_display();
+		CL_Display::flip_display();
+		CL_Display::clear_display();
+		CL_Display::flip_display();
+		CL_MouseCursor::hide();
 
 		show_fps = false;
 		key_F1 = false;
+
 #ifndef WIN32
 		map_path = "maps";
 		if (!opendir(map_path))
 		{
-			map_path = "/usr/local/share/clanbomber/maps";
+			map_path = PKGDATADIR "maps";
 			if (!opendir(map_path))
 			{
 				map_path = "/usr/share/clanbomber/maps";
@@ -205,7 +235,7 @@ int ClanBomberApplication::main(int argc, char** argv)
 	    	 			case MENU_EXIT:
      					menu->scroll_out();
      					delete menu;
-     					return 0;
+					quit_app(0);
 	     			break;
     		 			case MENU_PLAYER_SETUP:
 	     				menu->scroll_out();
@@ -392,7 +422,7 @@ int ClanBomberApplication::main(int argc, char** argv)
 	     				menu->scroll_in();
 	     			break;
 	     			default:
-	     				cout << result << endl;
+					std::cout << result << std::endl;
 	     			break;
 	     		}
 	     	}
@@ -411,14 +441,15 @@ int ClanBomberApplication::main(int argc, char** argv)
 			}
 		}
 
-		return 1; // fuck, shouldnt reach this
+		quit_app(1); // fuck shouldn't reach this
      }
      catch (CL_Error err)
      {
-		cout << endl << "Exception caught from ClanLib:" << endl;
-		cout << err.message << endl;
-		return 255;
+		std::cout << std::endl << "Exception caught from ClanLib:" << std::endl;
+		std::cout << err.message << std::endl;
+		quit_app(255);
      }
+	return 0;
 }
 
 void ClanBomberApplication::run_game()
@@ -429,11 +460,29 @@ void ClanBomberApplication::run_game()
 	CL_Display::flip_display();
 	CL_Display::clear_display();
 	CL_Display::flip_display();
+	CL_Display::clear_display();
+	CL_Display::flip_display();
 	
+	bool pause_game = false;
+	bool key_P = false;
+
 	while (CL_Keyboard::get_keycode(CL_KEY_ESCAPE) == 0)
 	{
 		Timer::time_elapsed(true);
 
+		if (CL_Keyboard::get_keycode(CL_KEY_P) && !key_P) {
+			pause_game = !pause_game;
+			if (pause_game) {
+				Resources::Font_big()->print_center(400, 300, "- PAUSED -");
+			}
+			CL_Display::flip_display();
+		}
+		key_P = CL_Keyboard::get_keycode(CL_KEY_P);
+		if (pause_game) {
+			CL_System::keep_alive();
+			continue;
+		}
+
 		if (observer != NULL)
 		{
 			observer->act();
@@ -444,9 +493,9 @@ void ClanBomberApplication::run_game()
 			}
 		}
 
+		delete_some();
 		act_all();
 		show_all();
-		delete_some();
 		
 		CL_Display::flip_display();
 		
@@ -514,8 +563,13 @@ void ClanBomberApplication::delete_some()
 }
 
 void ClanBomberApplication::show_all()
-{	
-	GameObject* draw_list[objects.get_num_items()+bomber_objects.get_num_items()]; 
+{
+	int max_draw_list_items = objects.get_num_items()+bomber_objects.get_num_items();
+#ifndef WIN32
+	GameObject* draw_list[max_draw_list_items];
+#else	
+	GameObject** draw_list = new GameObject*[max_draw_list_items];
+#endif
 
 	int n = 0;
 	int i;
@@ -584,6 +638,9 @@ void ClanBomberApplication::show_all()
 		observer->show();
 		CL_Display::set_clip_rect( CL_ClipRect(0, 0, 800, 600) );
 	}
+#ifdef WIN32
+	delete draw_list;
+#endif
 }
 
 void ClanBomberApplication::init_game()
@@ -596,7 +653,7 @@ void ClanBomberApplication::init_game()
 	{
 		try
 		{
-			wav = CL_Streamed_MikModSample::create("music/bud.mod", NULL, true);
+			wav = CL_Streamed_MikModSample::create(PKGDATADIR "bud.mod", NULL, true);
 			ses = new CL_SoundBuffer_Session( wav->prepare() );
 			ses->set_volume(1.2f);
 		}
@@ -654,9 +711,9 @@ void ClanBomberApplication::init_game()
 		if (Config::bomber[i].is_enabled())
 		{
 			pos = map->get_bomber_pos(j++);
-			new Bomber( pos.x*40, pos.y*40, (Bomber::COLOR)(Config::bomber[i].get_skin()), Controller::create((Controller::CONTROLLER_TYPE)Config::bomber[i].get_controller()), Config::bomber[i].get_name(), Config::bomber[i].get_team(),i, this );
+			new Bomber( int(pos.x*40), int(pos.y*40), (Bomber::COLOR)(Config::bomber[i].get_skin()), Controller::create((Controller::CONTROLLER_TYPE)Config::bomber[i].get_controller()), Config::bomber[i].get_name(), Config::bomber[i].get_team(),i, this );
 			bomber_objects.get_last()->set_pos( 350, 270 );
-			bomber_objects.get_last()->fly_to( pos.x*40, pos.y*40, 300 );
+			bomber_objects.get_last()->fly_to( int(pos.x*40), int(pos.y*40), 300 );
 			bomber_objects.get_last()->controller->deactivate();
 		}		
 	}
